package org.yasrun.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

/**
 * Excelファイル操作クラス<br/>
 * 参考1：http://www.javadrive.jp/poi/install/index3.html<br/>
 * 参考2：http://hondou.homedns.org/pukiwiki/pukiwiki.php?Jakarta%20POI<br/>
 * POI-3.7にて開発
 * @author yas
 */
public class ExcelFile {

    public static final int MAX_ROW = 65535;

    public static final int MAX_COLUMN = 256;

    private String filepath = null;

    private HSSFWorkbook book = null;

    private HSSFSheet sheet = null;

    private FileInputStream inStream = null;

    public ExcelFile(String path) {
        this.filepath = path;
    }

    public boolean selectSheet(int no) {
        boolean result = false;
        if (book != null) {
            try {
                sheet = book.getSheetAt(no);
                result = true;

            } catch (Exception e) {
                e.printStackTrace();

            }
        }
        return result;
    }

    public boolean selectSheet(String name) {
        boolean result = false;
        if (book != null) {
            try {
                sheet = book.getSheet(name);
                result = true;

            } catch (Exception e) {
                e.printStackTrace();

            }
        }
        return result;
    }

    public String getValue(int r, int c) {
        String result = null;

        if (r < 0 || r > MAX_ROW || c < 0 || c > MAX_COLUMN) {
            return result;
        }

        try {
            if (sheet != null) {
                final HSSFCell cell = sheet.getRow(r).getCell(c);
                result = getValueFromCell(cell);
            }

        } catch (Exception e) {
            e.printStackTrace();

        }

        return result;
    }

    private String getValueFromCell(HSSFCell cell) {
        String result = null;

        final int type = cell.getCellType();
        if (type == HSSFCell.CELL_TYPE_NUMERIC) {
            if (HSSFDateUtil.isCellDateFormatted(cell)) {
                result = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(cell.getDateCellValue());
            } else {
                result = Double.toString(cell.getNumericCellValue());
            }
        } else if (type == HSSFCell.CELL_TYPE_STRING) {
            result = cell.toString();
        }

        return result;
    }

    public boolean putValue(int r, int c, String value) {
        if (r < 0 || r > MAX_ROW || c < 0 || c > MAX_COLUMN) {
            return false;
        }

        boolean result = true;

        try {
            if (sheet != null) {
                HSSFRow row = sheet.getRow(r);
                if (row == null) {
                    row = sheet.createRow(r);
                }
                HSSFCell cell = row.getCell(c);
                if (cell == null) {
                    cell = row.createCell(c);
                }
                cell.setCellValue(value);
            }

        } catch (Exception e) {
            e.printStackTrace();
            result = false;

        }

        return result;
    }

    public String[] getRow(int r, int toColumn) {
        String[] result = null;

        if (r < 0 || r > MAX_ROW || toColumn < 1 || toColumn > MAX_COLUMN) {
            return result;
        }

        if (sheet != null) {
            List<String> list = new ArrayList<String>();

            for(int i = 0; i < toColumn; i++) {
                list.add(getValue(r, i));
            }

            result = list.toArray(new String[0]);
        }

        return result;
    }

    public boolean open() {
        boolean result = true;

        try {
            inStream = new FileInputStream(filepath);
            book = new HSSFWorkbook(inStream);

        } catch (FileNotFoundException e) {
            e.printStackTrace();
            result = false;

        } catch (IOException e) {
            e.printStackTrace();
            result = false;

        }

        return result;
    }

    public boolean save() {
        if (book == null) {
            book = new HSSFWorkbook();
            book.createSheet("Sheet1");
        }

        boolean result = true;

        final File file = new File(filepath);
        if (file.exists()) {
            file.delete();
        }

        FileOutputStream outStream = null;

        try {
            outStream = new FileOutputStream(filepath);
            book.write(outStream);

        } catch (FileNotFoundException e) {
            e.printStackTrace();
            result = false;

        } catch (IOException e) {
            e.printStackTrace();
            result = false;

        } finally {
            try {
                outStream.close();

            } catch (IOException e) {
                e.printStackTrace();
                result = false;

            }

        }

        return result;
    }

    public boolean saveAs(String path) {
        this.filepath = path;
        return save();
    }

    public void close() {
        this.sheet = null;
        this.book = null;

        try {
            if (inStream != null) {
                inStream.close();
                inStream = null;
            }

        } catch (IOException e) {
            e.printStackTrace();

        }
    }

    public void finalize() throws Throwable {
        super.finalize();
        close();
    }
}
