package org.yasrun.tools;

import java.io.File;

import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.MethodDoc;
import com.sun.javadoc.RootDoc;

/**
 * 単体テスト用 ドックレット<br/>
 * 参考:http://java.sun.com/j2se/1.5.0/ja/docs/ja/guide/javadoc/doclet/overview.
 * html
 *
 * @author yas
 */
public class UnitTestDoclet {

    public static boolean start(RootDoc root) {
        if (!Settings.getSettings()) {
            return false;
        }

        final ClassDoc[] classes = root.classes();

        int totalCount = 0;
        final StringBuilder sb = new StringBuilder();
        for (ClassDoc classDoc : classes) {
            int count = OutputClassInfo(classDoc);
            sb.append(classDoc.name() + "\t" + Integer.toString(count) + "\r\n");
            totalCount += count;
        }

        sb.append("計\t" + Integer.toString(totalCount) + "\r\n");

        final String summaryTextPath = Settings.outputDirPath + "summary.txt";
        FileUtil.savePlainText(sb.toString(), summaryTextPath);

        return true;
    }

    public static int OutputClassInfo(ClassDoc classDoc) {
        final String className = classDoc.name();
        int count = 0;

        if (className.endsWith("Test")) {
            final String targetClassname = className.substring(0, className.length() - 4);
            final String packageName = classDoc.containingPackage().name();
            final String dirpath = Settings.outputDirPath + packageName + "\\";
            final String templateFilepath = Settings.templatePath;
            final String filepath = dirpath + className + ".xls";

            new File(dirpath).mkdirs();
            final ExcelFile excel = new ExcelFile(templateFilepath);
            excel.open();
            excel.selectSheet(Settings.targetSheetName);

            final MethodDoc[] methods = classDoc.methods();
            for (MethodDoc methodDoc : methods) {
                count = OutputMethodInfo(methodDoc, count, excel);
            }

            excel.putValue(Settings.titleRow, Settings.titleColumn, targetClassname);
            excel.putValue(Settings.countRow, Settings.countColumn, Integer.toString(count));

            excel.saveAs(filepath);

            excel.close();
        }

        return count;
    }

    public static int OutputMethodInfo(MethodDoc methodDoc, int count, ExcelFile excel) {
        final String methodName = methodDoc.name();

        if (methodName.startsWith("test")) {
            String commentText = methodDoc.commentText();
            commentText = commentText.replaceAll("<br/>\r\n|<br/>\r|<br/>\n", "<br/>");
            final String[] comments = commentText.split("<br/>");

            final String no = Integer.toString(count + 1);
            final String title = comments.length < 1 ? "" : comments[0];
            final String howto = comments.length < 2 ? "" : comments[1];
            final String expected = comments.length < 3 ? "" : comments[2];
            final String dateString = Settings.getDateString();
            final String result = "OK";

            excel.putValue(count + Settings.startRow, 0, no);
            excel.putValue(count + Settings.startRow, 1, title + "\r\n(" + methodName + ")");
            excel.putValue(count + Settings.startRow, 2, howto);
            excel.putValue(count + Settings.startRow, 3, expected);
            excel.putValue(count + Settings.startRow, 4, dateString);
            excel.putValue(count + Settings.startRow, 5, result);

            count++;
        }

        return count;
    }
}
